@@ -170,7 +170,7 @@ socket_t udp_create_socket(const udp_socket_config_t *config) {
 
 error:
 	freeaddrinfo(ai_list);
-	if(sock != INVALID_SOCKET)
+	if (sock != INVALID_SOCKET)
 		closesocket(sock);
 
 	return INVALID_SOCKET;
@@ -217,11 +217,40 @@ int udp_sendto(socket_t sock, const char *data, size_t size, const addr_record_t
 #endif
 }
 
+int udp_sendto_self(socket_t sock, const char *data, size_t size) {
+	addr_record_t local;
+	if (udp_get_local_addr(sock, AF_UNSPEC, &local) < 0)
+		return -1;
+
+	int ret;
+#ifndef __linux__
+	// We know local has the same address family as sock here
+	ret = sendto(sock, data, (socklen_t)size, 0, (const struct sockaddr *)&local.addr, local.len);
+#else
+	ret = sendto(sock, data, size, 0, (const struct sockaddr *)&local.addr, local.len);
+#endif
+	if (ret >= 0 || local.addr.ss_family != AF_INET6)
+		return ret;
+
+	// Fallback as IPv6 may be disabled on the loopback interface
+	if (udp_get_local_addr(sock, AF_INET, &local) < 0)
+		return -1;
+
+#ifndef __linux__
+	addr_map_inet6_v4mapped(&local.addr, &local.len);
+	return sendto(sock, data, (socklen_t)size, 0, (const struct sockaddr *)&local.addr, local.len);
+#else
+	return sendto(sock, data, size, 0, (const struct sockaddr *)&local.addr, local.len);
+#endif
+}
+
 int udp_set_diffserv(socket_t sock, int ds) {
 #ifdef _WIN32
 	// IP_TOS has been intentionally broken on Windows in favor of a convoluted proprietary
 	// mechanism called qWave. Thank you Microsoft!
 	// TODO: Investigate if DSCP can be still set directly without administrator flow configuration.
+	(void)sock;
+	(void)ds;
 	JLOG_INFO("IP Differentiated Services are not supported on Windows");
 	return -1;
 #else
@@ -284,10 +313,14 @@ int udp_get_local_addr(socket_t sock, int family_hint, addr_record_t *record) {
 		return -1;
 
 	// If the socket is bound to a particular address, return it
-	if(!addr_is_any((struct sockaddr *)&record->addr))
-		return 0;
+	if (!addr_is_any((struct sockaddr *)&record->addr)) {
+		if (record->addr.ss_family == AF_INET && family_hint == AF_INET6)
+			addr_map_inet6_v4mapped(&record->addr, &record->len);
 
-	if(record->addr.ss_family == AF_INET6 && family_hint == AF_INET) {
+		return 0;
+	}
+
+	if (record->addr.ss_family == AF_INET6 && family_hint == AF_INET) {
 		// Generate an IPv4 instead (socket is listening to any IPv4 or IPv6)
 
 		uint16_t port = addr_get_port((struct sockaddr *)&record->addr);
@@ -319,6 +352,10 @@ int udp_get_local_addr(socket_t sock, int family_hint, addr_record_t *record) {
 		// Ignore
 		break;
 	}
+
+	if (record->addr.ss_family == AF_INET && family_hint == AF_INET6)
+		addr_map_inet6_v4mapped(&record->addr, &record->len);
+
 	return 0;
 }
 
@@ -428,7 +465,9 @@ int udp_get_addrs(socket_t sock, addr_record_t *records, size_t count) {
 	for (int i = 0; i < list->iAddressCount; ++i) {
 		struct sockaddr *sa = list->Address[i].lpSockaddr;
 		socklen_t len = list->Address[i].iSockaddrLength;
-		if ((sa->sa_family == AF_INET || sa->sa_family == AF_INET6) && !addr_is_local(sa) &&
+		if ((sa->sa_family == AF_INET ||
+		     (sa->sa_family == AF_INET6 && bound.addr.ss_family == AF_INET6)) &&
+		    !addr_is_local(sa) &&
 		    !(has_temp_inet6 && sa->sa_family == AF_INET6 && !addr_is_temp_inet6(sa))) {
 			if (!has_duplicate_addr(sa, records, current - records)) {
 				++ret;
@@ -473,7 +512,10 @@ int udp_get_addrs(socket_t sock, addr_record_t *records, size_t count) {
 
 		struct sockaddr *sa = ifa->ifa_addr;
 		socklen_t len;
-		if (sa && (sa->sa_family == AF_INET || sa->sa_family == AF_INET6) && !addr_is_local(sa) &&
+		if (sa &&
+		    (sa->sa_family == AF_INET ||
+		     (sa->sa_family == AF_INET6 && bound.addr.ss_family == AF_INET6)) &&
+		    !addr_is_local(sa) &&
 		    !(has_temp_inet6 && sa->sa_family == AF_INET6 && !addr_is_temp_inet6(sa)) &&
 		    (len = addr_get_len(sa)) > 0) {
 			if (!has_duplicate_addr(sa, records, current - records)) {
@@ -520,42 +562,44 @@ int udp_get_addrs(socket_t sock, addr_record_t *records, size_t count) {
 		}
 	}
 
-	char hostname[HOST_NAME_MAX];
-	if (gethostname(hostname, HOST_NAME_MAX))
-		strcpy(hostname, "localhost");
+	if (bound.addr.ss_family == AF_INET6) {
+		char hostname[HOST_NAME_MAX];
+		if (gethostname(hostname, HOST_NAME_MAX))
+			strcpy(hostname, "localhost");
 
-	char service[8];
-	snprintf(service, 8, "%hu", port);
+		char service[8];
+		snprintf(service, 8, "%hu", port);
 
-	struct addrinfo *ai_list = NULL;
-	struct addrinfo hints;
-	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = AF_INET6;
-	hints.ai_socktype = SOCK_DGRAM;
-	hints.ai_protocol = 0;
-	hints.ai_flags = AI_ADDRCONFIG | AI_NUMERICSERV;
-	if (getaddrinfo(hostname, service, &hints, &ai_list) == 0) {
-		bool has_temp_inet6 = false;
-		for (struct addrinfo *ai = ai_list; ai; ai = ai->ai_next) {
-			if (addr_is_temp_inet6(ai->ai_addr)) {
-				has_temp_inet6 = true;
-				break;
+		struct addrinfo *ai_list = NULL;
+		struct addrinfo hints;
+		memset(&hints, 0, sizeof(hints));
+		hints.ai_family = AF_INET6;
+		hints.ai_socktype = SOCK_DGRAM;
+		hints.ai_protocol = 0;
+		hints.ai_flags = AI_ADDRCONFIG | AI_NUMERICSERV;
+		if (getaddrinfo(hostname, service, &hints, &ai_list) == 0) {
+			bool has_temp_inet6 = false;
+			for (struct addrinfo *ai = ai_list; ai; ai = ai->ai_next) {
+				if (addr_is_temp_inet6(ai->ai_addr)) {
+					has_temp_inet6 = true;
+					break;
+				}
 			}
-		}
-		for (struct addrinfo *ai = ai_list; ai; ai = ai->ai_next) {
-			if (!addr_is_local(ai->ai_addr) &&
-			    !(has_temp_inet6 && !addr_is_temp_inet6(ai->ai_addr))) {
-				if (!has_duplicate_addr(ai->ai_addr, records, current - records)) {
-					++ret;
-					if (current != end) {
-						memcpy(&current->addr, ai->ai_addr, ai->ai_addrlen);
-						current->len = ai->ai_addrlen;
-						++current;
+			for (struct addrinfo *ai = ai_list; ai; ai = ai->ai_next) {
+				if (!addr_is_local(ai->ai_addr) &&
+				    !(has_temp_inet6 && !addr_is_temp_inet6(ai->ai_addr))) {
+					if (!has_duplicate_addr(ai->ai_addr, records, current - records)) {
+						++ret;
+						if (current != end) {
+							memcpy(&current->addr, ai->ai_addr, ai->ai_addrlen);
+							current->len = ai->ai_addrlen;
+							++current;
+						}
 					}
 				}
 			}
+			freeaddrinfo(ai_list);
 		}
-		freeaddrinfo(ai_list);
 	}
 #endif
 #endif
